#!/usr/bin/env python3
"""Module with technical alphas"""
import stbt.operators.technical as ti
import stbt.alphas.events as ev
from stbt.helpers import prolong, unpack_ohlc


def acceleration_event(ohlc_df, lookback=60, prolong_timeframe=240, threshold=0.1):
    '''alpha based of long term acceleration
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del volume, opens, high, low
    events = ev.get_acc_events(close, lookback, threshold)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    weights.fillna(0, inplace=True)
    return weights


def vwap_event(ohlc_df,
               threshold=.01, vwap_timeframe=60,
               prolong_timeframe=240,
               multiplier=1):
    '''alpha based on close being too far from vwap
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del opens, high, low

    events = ev.get_vwap_events(close, volume, threshold, vwap_timeframe)
    weights = events.rolling(prolong_timeframe).apply(prolong)

    return weights * multiplier


def high_volume_small_candle_event(ohlc_df,
                                   lookback=60,
                                   threshold_volume=10,
                                   threshold_candle=1.5,
                                   prolong_timeframe=240,
                                   multiplier=1):
    '''alpha based on high volume with in the small candle
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low

    volume_events = ev.get_high_volume_events(
        volume, lookback, threshold_volume)
    candle_events = ev.get_small_candle_body_events(
        close, opens, threshold_candle, lookback)
    events = volume_events.multiply(candle_events)
    weights = events.rolling(prolong_timeframe).apply(prolong)

    return weights * multiplier


def high_volume_big_candle_event(ohlc_df,
                                 lookback=60,
                                 threshold_volume=10,
                                 threshold_candle=.01,
                                 prolong_timeframe=240,
                                 multiplier=1,
                                 delay=None):
    '''alpha based on high volume with in the big candle
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low

    volume_events = ev.get_high_volume_events(
        volume, lookback, threshold_volume)
    candle_events = ev.get_big_candle_body_events(
        close, opens, threshold_candle)
    events = volume_events.multiply(candle_events)
    weights = events.rolling(prolong_timeframe).apply(prolong)

    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def high_volume_big_tail_event(ohlc_df,
                               lookback=60,
                               threshold_volume=10,
                               threshold_tail=1.005,
                               prolong_timeframe=240,
                               multiplier=1,
                               delay=None):
    '''alpha based on high volume with in the big candle
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)

    volume_events = ev.get_high_volume_events(
        volume, lookback, threshold_volume)
    candle_events = ev.get_big_tail_events(
        close, high, low, opens, threshold_tail)
    events = volume_events.multiply(candle_events)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def high_volume_vwap_event(ohlc_df,
                           lookback_volume=1000,
                           threshold_volume=10,
                           threshold_vwap=1.005,
                           vwap_timeframe=60,
                           prolong_timeframe=240,
                           multiplier=1,
                           delay=None):
    '''alpha based on high volume and price being far from vwap
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low, opens
    volume_events = ev.get_high_volume_events(
        volume, lookback_volume, threshold_volume)
    vwap_events = ev.get_vwap_events(
        close, volume, threshold_vwap, vwap_timeframe)
    events = volume_events.multiply(vwap_events)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def multiple_tails_event(ohlc_df,
                         threshold_tail=1.005,
                         count_timeframe=60,
                         tail_count=2,
                         prolong_timeframe=240,
                         multiplier=1,
                         delay=None):
    '''alpha based on multiple big tails in small count_timeframe
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del volume
    candle_events = ev.get_big_tail_events(
        close, high, low, opens, threshold_tail)
    events = ev.get_event_count_events(
        candle_events, tail_count, count_timeframe)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def multiple_high_volume_small_candle_event(ohlc_df,
                                            lookback_volume=60,
                                            threshold_volume=10,
                                            threshold_candle=1.5,
                                            count_timeframe=60,
                                            event_count=2,
                                            prolong_timeframe=240,
                                            multiplier=1,
                                            delay=None):
    '''alpha based on multiple big tails in small count_timeframe
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low

    volume_events = ev.get_high_volume_events(
        volume, lookback_volume, threshold_volume)
    candle_events = ev.get_small_candle_body_events(
        close, opens, threshold_candle, lookback_volume)
    events_hvsc = volume_events.multiply(candle_events)
    events = ev.get_event_count_events(
        events_hvsc, event_count, count_timeframe)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def multiple_vwap_event(ohlc_df,
                        threshold=.01,
                        vwap_timeframe=60,
                        count_timeframe=60,
                        event_count=2,
                        prolong_timeframe=240,
                        multiplier=1,
                        delay=None):
    '''alpha based on multiple vwap events
    '''

    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del opens, high, low

    vwap_events = ev.get_vwap_events(close, volume, threshold, vwap_timeframe)
    events = ev.get_event_count_events(
        vwap_events, event_count, count_timeframe)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier


def multiple_high_volume_small_candle_event_time(ohlc_df,
                                                 lookback_volume=60,
                                                 threshold_volume=10,
                                                 threshold_candle=1.5,
                                                 count_timeframe=60,
                                                 event_count=2,
                                                 prolong_timeframe=240,
                                                 multiplier=1,
                                                 start_time="10:00",
                                                 end_time="10:30",
                                                 delay=None):
    '''alpha based on multiple big tails in small count_timeframe
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low

    volume_events = ev.get_high_volume_events(
        volume, lookback_volume, threshold_volume)
    candle_events = ev.get_small_candle_body_events(
        close, opens, threshold_candle, lookback_volume)
    events_hvsc = volume_events.multiply(candle_events)
    events = ev.get_event_count_events(
        events_hvsc, event_count, count_timeframe)
    events = ti.null_daytime_timeperiod(events, start_time, end_time)
    weights = events.rolling(prolong_timeframe).apply(prolong)
    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier

def low_volume_big_candle_event(ohlc_df,
                                lookback=60,
                                threshold_volume=0,
                                threshold_candle=.01,
                                prolong_timeframe=240,
                                multiplier=1,
                                delay=None):
    '''alpha based on high volume with in the big candle
    '''
    volume, opens, high, low, close = unpack_ohlc(ohlc_df)
    del high, low

    volume_events = ev.get_low_volume_events(
        volume, lookback, threshold_volume)
    candle_events = ev.get_big_candle_body_events(
        close, opens, threshold_candle)
    events = volume_events.multiply(candle_events)
    weights = events.rolling(prolong_timeframe).apply(prolong)

    if delay:
        weights = ti.delay(weights, delay)

    return weights * multiplier



#!/usr/bin/env python3
"""Module with technical operators/indicators"""
import numpy as np
import pandas as pd
from stbt.helpers import string_to_time


def vwap(close, volume, vwaptimeframe=60):
    """Self explanatory"""
    vol60_df = volume.rolling(vwaptimeframe).sum()
    vol60_df.fillna(0, inplace=True)
    close_vol_mul_df = volume*close
    close_vol_cumsum_df = close_vol_mul_df.rolling(vwaptimeframe).sum()
    close_vol_cumsum_df.fillna(0, inplace=True)
    vwap_df = close_vol_cumsum_df/vol60_df

    return vwap_df


def simple_sign(ohlc_df):
    """Self explanatory"""
    df = ohlc_df.copy()
    df[df > 0] = 1
    df[df < 0] = -1
    df.fillna(0)
    return df


def custom_rank(array):
    """Self explanatory"""
    s = pd.Series(array)
    return s.rank(ascending=False)[len(s)-1]


def normalize(df_ohlc, timeframe=60):
    """Self explanatory"""
    return df_ohlc/(df_ohlc.rolling(timeframe).max())


def neutralize(weights, axis=1):
    """https://realmoney.thestreet.com/articles/06/28/2016/portfolios-should-be-neutralized"""

    weights = weights.sub(weights.mean(axis=1), axis=0)

    return weights


def delay(weights, delay=1):
    """https://realmoney.thestreet.com/articles/06/28/2016/portfolios-should-be-neutralized"""

    weights = weights.shift(delay)

    return weights


def delta(df_ohlc, periods=5):
    """Self explanatory"""
    df_ohlc = df_ohlc.diff(periods)

    return df_ohlc


def acc(closes):
    """Self explanatory"""
    acc = (closes - 2 * delay(closes, 1) + delay(closes, 2))/delay(closes, 2)
    # acc = close_minus_open.pct_change()

    return acc


def acceleration(df_ohlc, periods=24):
    """Self explanatory"""
    slopes = df_ohlc.pct_change()
    df_ohlc = regr(slopes, periods)

    return df_ohlc


def argmax(df_ohlc, periods=24):
    """Self explanatory"""

    return df_ohlc.rolling(periods).apply(np.argmax)


def argmin(df_ohlc, periods=24):
    """Self explanatory"""

    return df_ohlc.rolling(periods).apply(np.argmin)


def regr(df_ohlc, periods=24):
    """https://en.wikipedia.org/wiki/Linear_regression"""
    df_ohlc = df_ohlc.rolling(periods).apply(lambda x: np.cov(x, range(periods))[0][1]
                                             / np.std(range(periods))**2)

    return df_ohlc


def sign(df_ohlc, low_zero, high_zero):
    """Assigns 0, -1, 1 depending on sign of value"""
    zero_labels = (df_ohlc > low_zero) & (df_ohlc < high_zero)
    minus_labels = df_ohlc <= low_zero
    plus_labels = df_ohlc >= high_zero
    df_ohlc[zero_labels] = 0
    df_ohlc[minus_labels] = -1
    df_ohlc[plus_labels] = 1

    return df_ohlc


def skewness(df_ohlc, periods=24):
    """https://en.wikipedia.org/wiki/Skewness_risk"""

    return df_ohlc.rolling(periods).skew()


def standard_deviation(df_ohlc, periods=24):
    """Self explanatory"""
    df_ohlc = df_ohlc.rolling(periods, min_periods=periods).std()

    return df_ohlc


def ts_rank(df_ohlc, periods=60):
    """Self explanatory"""

    return df_ohlc.rolling(periods).apply(custom_rank)


def scale(weights):
    """Self explanatory"""

    return weights.div(weights.abs().sum(axis=1), axis=0)


def rank(weights):
    """Self explanatory"""

    return weights.rank(axis=1).div(weights.rank(axis=1).max(axis=1), axis=0)


def hump(weights, change_threshold=1):
    """Self explanatory"""
    previous_row = None
    previous_index = None
    for index, row in weights.iterrows():
        if previous_index:
            if (row - previous_row).abs().sum()/max(row.abs().sum(), 1) < change_threshold:
                weights.loc[index, :] = previous_row
        previous_row = row
        previous_index = index

    return weights


def null_noise(df_ohlc, low_zero=-0.01, high_zero=0.01):
    """Assigns 0, -1, 1 depending on sign of value"""
    zero_labels = (df_ohlc > low_zero) & (df_ohlc < high_zero)
    df_ohlc[zero_labels] = 0

    return df_ohlc


def null_daytime_timeperiod(df_weights, start="10:00", end="10:30"):
    """Assigns 0 if df.index is in e.g. from 10:00 to 10:30"""
    indexes = (df_weights.index.time > string_to_time(start)) &\
        (df_weights.index.time < string_to_time(end))
    df_weights[indexes] = 0

    return df_weights



#!/usr/bin/env python3
"""Module with Strategy class to all
   backtest related manipulations"""
import datetime as dt
import pandas as pd
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.ticker import MaxNLocator
matplotlib.use('TkAgg')

def string_to_time(time_str):
    '''From 14:32 to datetime.time()'''
    if len(time_str) != 5:
        return dt.time(0, 0)
    else:
        if time_str[3] != '0':
            minutes = int(time_str[3:])
        else:
            minutes = int(time_str[4:])
        if time_str[0] != '0':
            hours = int(time_str[:2])
        else:
            hours = int(time_str[:1])
        return dt.time(hours, minutes)

def plot_charts(df_ohlc):
    """Function to plot simple Close/Volume graph

    Args:
        df_ohlc (DataFrame):
            Close and Volume columns are necessary

    Returns:
        figure (figure):
            Matplotlib graph
    """
    if str(type(df_ohlc.index[0])) == "<class 'pandas._libs.tslib.Timestamp'>":
        pass
    else:
        df_ohlc.index = pd.to_datetime(df_ohlc.index)

    figure = plt.figure()

    ax1 = plt.subplot2grid((8, 1), (0, 0), rowspan=5, colspan=1)
    ax1.plot(df_ohlc.index, df_ohlc['Close'], label='Close')

    plt.xlabel('Date')
    plt.ylabel('Price')
    plt.title('Сharts')

    ax2 = plt.subplot2grid((8, 1), (6, 0), rowspan=2, colspan=1, sharex=ax1)
    ax2.plot(df_ohlc.index, df_ohlc['Volume'], label='Volume')

    plt.xticks(rotation=45)
    ax1.xaxis.set_major_locator(MaxNLocator(5))

    plt.ylabel('Volume')

    return figure


def resample(df_ohlc, frequency='H'):
    """Function to change frequency and fill the gaps"""
    df_ohlc = df_ohlc.resample(frequency).ffill()
    df_ohlc = df_ohlc.fillna(0)

    return df_ohlc

def get_sharpe(df_ohlc):
    """Function to calculate sharpe ratio"""
    if isinstance(df_ohlc, pd.DataFrame):
        return round((np.sqrt(len(df_ohlc)) * df_ohlc.mean() / df_ohlc.std().values).values[0], 2)
    else:
        return round((np.sqrt(len(df_ohlc)) * df_ohlc.mean() / df_ohlc.std()), 2)


def get_max_drawdown(returns):
    """Assumes returns is a pandas Series"""
    ret = returns.add(1).cumprod()
    daily_drawdown = ret.div(ret.cummax()).sub(1)
    max_drawdown = round(daily_drawdown.min(), 4)
    end = daily_drawdown.idxmin()
    start = ret.loc[:end].idxmax()
    return max_drawdown, start, end

def get_label_from_dict(settings_dict):
    """Function to get name from dict"""
    label = ''
    if 'delay' in settings_dict.keys():
        label += 'd' + str(settings_dict['delay'])
    if 'commissions_const' in settings_dict.keys():
        label += '_' + str(settings_dict['commissions_const'] * 100) + '%' + 'coms'

    return label

def initialize_universe(ohlc_df):
    '''func to create technical data dfs and add them to dict'''
    # add moex_stocks Universe
    uni_dict = {}
    # ohlc_df = pd.read_csv(DATA_DIR + csv_file, index_col='Date')
    ohlc_df.index = pd.to_datetime(ohlc_df.index)
    ohlc_df.sort_index(inplace=True)
    ohlc_df = ohlc_df.loc[~ohlc_df.index.duplicated(keep='first')]
    # print(ohlc_df.info(max_cols=1000, null_counts=True))
    high_cols = [column for column in ohlc_df.columns if 'High' in column]
    low_cols = [column for column in ohlc_df.columns if 'Low' in column]
    close_cols = [column for column in ohlc_df.columns if 'Close' in column]
    open_cols = [column for column in ohlc_df.columns if 'Open' in column]
    volume_cols = [column for column in ohlc_df.columns if 'Volume' in column]

    uni_dict['close'] = ohlc_df[close_cols].rename(columns=lambda col: col.replace('Close_', ''))
    uni_dict['volume'] = ohlc_df[volume_cols].rename(columns=lambda col: col.replace('Volume_', ''))
    uni_dict['open'] = ohlc_df[open_cols].rename(columns=lambda col: col.replace('Open_', ''))
    uni_dict['low'] = ohlc_df[low_cols].rename(columns=lambda col: col.replace('Low_', ''))
    uni_dict['high'] = ohlc_df[high_cols].rename(columns=lambda col: col.replace('High_', ''))
    uni_dict['all'] = ohlc_df

    return uni_dict

# def prolong(series):
#     '''func to prolong weight
#     weights_df = events.rolling(prolong_timeframe).apply(prolong)'''
#     new_ser = series.copy()
#     # if new_ser[0] == 0 and new_ser.sum() != 0:
#     if new_ser.sum() > 0 and new_ser.sum() != len(new_ser):
#         return 1
#     elif new_ser.sum() < 0 and new_ser.sum() != -len(new_ser):
#         return -1
#     else:
#         return 0

def prolong(series):
    '''func to prolong weight
    weights_df = events.rolling(prolong_timeframe).apply(prolong)'''
    new_ser = series.copy()
    # if new_ser[0] == 0 and new_ser.sum() != 0:
    non_zero = new_ser[new_ser != 0]
    if not non_zero.empty:
        if non_zero[-1] > 0 and non_zero.sum() != len(new_ser):
            return 1
        elif non_zero[-1] < 0 and non_zero.sum() != -len(new_ser):
            return -1
        else:
            return 0
    else:
        return 0

def unpack_ohlc(ohlc_df):
    '''Just helper to create dict with all tech vars'''

    high_cols = [column for column in ohlc_df.columns if 'High' in column]
    low_cols = [column for column in ohlc_df.columns if 'Low' in column]
    close_cols = [column for column in ohlc_df.columns if 'Close' in column]
    open_cols = [column for column in ohlc_df.columns if 'Open' in column]
    volume_cols = [column for column in ohlc_df.columns if 'Volume' in column]

    close = ohlc_df[close_cols].rename(columns=lambda col: col.replace('Close_', ''))
    volume = ohlc_df[volume_cols].rename(columns=lambda col: col.replace('Volume_', ''))
    opens = ohlc_df[open_cols].rename(columns=lambda col: col.replace('Open_', ''))
    low = ohlc_df[low_cols].rename(columns=lambda col: col.replace('Low_', ''))
    high = ohlc_df[high_cols].rename(columns=lambda col: col.replace('High_', ''))

    return volume, opens, high, low, close



#!/usr/bin/env python3
"""Module show package usage"""

import datetime as dt
import pandas as pd
import matplotlib.pyplot as plt
from stbt.simulator import Strategy
from stbt.download_ohlc.cryptocurrency import get_ohlc_cryptocompare
from stbt.operators.technical import skewness

# get trading data from cryptocompare
BTC_TICKER = 'BTC'
ETH_TICKER = 'ETH'
USD_TICKER = 'USD'

END_DATE = dt.datetime(2018, 7, 1, 0, 0, 0)
START_DATE = dt.datetime(2018, 3, 1, 0, 0, 0)

OHLC_BTC = get_ohlc_cryptocompare(BTC_TICKER, USD_TICKER, START_DATE,
                                  end_date=END_DATE, interval_key='day')
OHLC_ETH = get_ohlc_cryptocompare(ETH_TICKER, USD_TICKER, START_DATE,
                                  end_date=END_DATE, interval_key='day')

# create dfs in format that Strategy requires
closes_df = pd.concat([OHLC_BTC['Close'], OHLC_ETH['Close']],
                      axis=1, keys=['BTC', 'ETH'])

# use imported indicator to create weights
weights_df = skewness(closes_df)

# create strategy
s = Strategy(closes_df, weights_df, cash=100)

# run backtest, robust tests, calculate stats
s.run_all(delay=2, verify_data_integrity=True, instruments_drop=None,
          commissions_const=0, capitalization=False, plot='all')

# check strategy stats
print(s.stats_dict)

# save strategy to futher comparison
s.add_to_pnls_pool()

# plot pool correlation heatmap
heatmap_fig, corr_matrix = s.get_pool_heatmap()

plt.show()



#!/usr/bin/env python3
"""Module with events"""
import numpy as np
import pandas as pd
import stbt.operators.technical as ti


def get_acc_events(close, lookback=60, threshold=0.1):
    '''event of acceleration being positive or negative for a long time
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)
    threshold is prevailance of positive acc against negative 0.1 = 10% more

    NOTE: try doing acc sum without sign
    '''
    acc_df = ti.acc(close)
    acc_sign_sum = acc_df.applymap(np.sign).rolling(window=lookback).sum()
    events = ti.sign(acc_sign_sum, -1*threshold*lookback, threshold*lookback)

    return events


###############################################################################
# volume events
###############################################################################

def get_vwap_events(close, volume, threshold=.01, vwap_timeframe=60):
    '''event of close being to far from vwap
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''

    vwap = ti.vwap(close, volume, vwap_timeframe)
    events = ti.sign((vwap - close)/(close+vwap)*2, -1*threshold, threshold)

    return events

def get_high_volume_events(volume, lookback=60, threshold=10):
    '''event of volume being too high
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    vol60_df = volume.rolling(lookback).mean()
    vol60_df.fillna(0, inplace=True)
    events = ti.sign((volume - vol60_df)/(vol60_df+1), -1*threshold, threshold)

    return events

def get_low_volume_events(volume, lookback=60, threshold=0):
    '''event of volume being lesser than average,
    threshold=0 stands for equal to average, -1 - 2*average
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    vol60_df = volume.rolling(lookback).mean()
    vol60_df.fillna(0, inplace=True)
    events = ti.sign((vol60_df - volume)/(vol60_df+1), -9999999999, threshold)

    return events

###############################################################################
# candle type events
###############################################################################
def get_big_candle_body_events(close, opens, threshold=.01):
    '''event of large abs(close-opens)
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''

    events = ti.sign((close-opens)/(close+opens)*2, -1*threshold, threshold)

    return events

def get_small_candle_body_events(close, opens, threshold=1.5, lookback=60):
    '''event of candle being no larger than ave_candle * threshold
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    average_candle = (close - opens).abs().rolling(lookback).mean()
    pos_ind = ((close-opens) >= 0) & ((close-opens) < average_candle * threshold)
    neg_ind = ((close-opens) < 0) & ((close-opens) < average_candle * threshold)
    # zero_ind = (close-opens) >= average_candle * threshold
    events = close * 0
    events[pos_ind] = 1
    events[neg_ind] = -1

    return events

def get_small_candle_events(high, low, threshold=1.5, lookback=60):
    '''event of candle being no larger than ave_candle * threshold
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    average_candle = (high - low).abs().rolling(lookback).mean()
    pos_ind = ((high-low) >= 0) & ((high-low) < average_candle * threshold)

    events = high * 0
    events[pos_ind] = 1

    return events

def get_big_tail_events(close, high, low, open_prices, threshold=1.005):
    '''event of large tail
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    high_tail_df = high - pd.concat([close, open_prices]).max(level=0)
    low_tail_df = - low + pd.concat([close, open_prices]).min(level=0)

    pos_events = ti.sign(high_tail_df/high + 1, -1*threshold, threshold)
    neg_events = ti.sign(low_tail_df/high - 1, -1*threshold, threshold)
    events = pos_events + neg_events

    return events

###############################################################################
# other events
###############################################################################

def get_event_count_events(events_df, count, timeframe):
    '''event of sum of events over timeframe being bigger than count
    or lesser than -count
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''
    events_sum = events_df.rolling(timeframe).sum()
    pos_ind = events_sum >= count
    neg_ind = events_sum <= - count
    events = events_sum * 0
    events[pos_ind] = 1
    events[neg_ind] = -1

    return events

def get_trend_events(volume, threshold=1.01, lookback=60):
    '''event of slope being bigger than threshold
    use afterwards as:
    result_df = events.rolling(prolong_timeframe).apply(prolong)'''

    slopes = volume.rolling(lookback).apply(lambda x: \
        np.polyfit(volume.index, volume/volume[-1], 1)[0])
    pos_ind = slopes >= threshold
    neg_ind = slopes <= - threshold
    events = volume * 0
    events[pos_ind] = 1
    events[neg_ind] = -1

    return events



#!/usr/bin/env python3
"""Module with all the tests"""

import unittest
import datetime as dt
import pandas as pd
from stbt.helpers import get_sharpe, get_max_drawdown
from stbt.simulator import Strategy

class TestStrategy(unittest.TestCase):
    """Class to test all Strategy methods"""
    @classmethod
    def setUpClass(cls):
        """Prepare data for futher tests"""
        cls.sim_results = {}

        # prepare datetime index
        some_date = dt.datetime(2017, 1, 1)
        days = pd.date_range(some_date, some_date + dt.timedelta(30), freq='D')

        # initialize close prices
        data_values = list(range(1, 32))
        cls.closes_df = pd.DataFrame({'Date': days, 'inst1': data_values})
        cls.closes_df.set_index('Date', inplace=True)

        # initialize weights
        weights_values = [1 for i in range(31)]
        cls.weights_df = pd.DataFrame({'Date': days, 'inst1': weights_values})
        cls.weights_df.set_index('Date', inplace=True)

        # create strategy
        cls.test_strategy = Strategy(cls.closes_df, cls.weights_df, cash=100)

        cls.sim_results = cls.test_strategy.backtest()

    def test_constructor(self):
        """Test to check Strategy __init__ method"""
        assert len(self.test_strategy.weights) == len(self.weights_df)

    def test_verify_data_integrity(self):
        """Test to check Strategy verify_data_integrity method
        P.S. method will raise error if data is not okay"""
        self.test_strategy.verify_data_integrity()

    def test_backtest(self):
        """Test to check Strategy backtest method"""
        assert self.sim_results['pnl'].values[2] == 250

    def test_plot_sim_results(self):
        """Test to check Strategy verify_data_integrity method
        P.S. method will raise error if data is not okay"""
        self.test_strategy.plot_sim_results(self.sim_results['pnl'])
        assert str(self.test_strategy.strategy_figure) == "Figure(640x480)"

    def test_calculate_sim_stats(self):
        """Test to check Strategy calculate_sim_stats method"""
        sim_stats_dict = self.test_strategy.calculate_sim_stats(self.sim_results['pnl'],
                                                                self.sim_results['returns'])
        assert sim_stats_dict['Avg_returns'] == "12.89%"

    def test_run_tests(self):
        """Test to check Strategy run_tests method"""
        self.test_strategy.run_tests()
        assert str(self.test_strategy.tests_figure) == "Figure(640x480)"

    def test_get_sharpe(self):
        """Test to check get_sharpe function"""
        test_df = pd.DataFrame({'pnl':[1, 2, 3]})
        assert get_sharpe(test_df) == 3.46

    def test_get_max_drawdown(self):
        """Test to check get_max_drawdown function"""
        test_df = pd.Series([0.01, -0.02, 0.03])
        drawdown_dict = get_max_drawdown(test_df)
        assert round(drawdown_dict[0], 2) == -0.02



#!/usr/bin/env python3
"""Module to download cryptocurrency ohlc data"""

import time
import datetime as dt
import logging
import json
import requests
import pandas as pd

def from_datetime_to_unix(date):
    '''in: datetime, out: unix_timestamp'''
    return int(time.mktime(date.timetuple()))

def from_unix_to_date(date):
    '''in: unix_timestamp, out: datetime'''
    value = dt.datetime.fromtimestamp(date)
    return value.date()

def str_to_datetime(time_str):
    '''in: str, out: datetime'''
    return dt.datetime.strptime(time_str, '%Y-%m-%d %H:%M:%S')


def columns_to_upper_case(df_ohlc):
    '''in : df, out : df, Makes all columns of df start with capital letter'''

    columns = list(df_ohlc.columns)
    for column in columns:
        if column[0].isupper():
            pass
        else:
            tmp_column_name = column[0].upper() + column[1:]
            df_ohlc.rename(index=str, columns={column: tmp_column_name}, inplace=True)
    return df_ohlc


def correct_ohlc_df(df_ohlc, frequency=None, cols_to_drop=None):
    """Function to modify df to the required format, checking for wrong entries
    and filling nans

    Args:
        df_ohlc (DataFrame):
            Close, Open, Low, High and Volume columns are necessary
        frequency (str):
            Resample frequency 'D', 'W', 'M', if None - do not resample data
        cols_to_drop (list):
            names of unnecessary columns in df

    Returns:
        df (DataFrame):
            Repaired df
    """
    if cols_to_drop is None:
        cols_to_drop = []

    if str(type(df_ohlc.index[0])) == "<class 'pandas._libs.tslib.Timestamp'>":
        pass
    else:
        df_ohlc.index = pd.to_datetime(df_ohlc.index)

    # resampling data if needed
    if frequency is not None:
        df_ohlc = df_ohlc.resample(frequency).agg({
            'Close': 'last',
            'High': 'max',
            'Low': 'min',
            'Open': 'first',
            'Volume': 'sum',
        })

    df_before_correction = df_ohlc

    # make ohlc right
    count_of_ohlc_mistakes = 0
    for index, row in df_ohlc.iterrows():
        if row['Low'] > min(row['Close'], row['Open'], row['High']):
            df_ohlc.loc[index, 'Low'] = min(row['Close'], row['Open'], row['High']) * 0.999
            count_of_ohlc_mistakes += 1
        if row['High'] < max(row['Close'], row['Open'], row['Low']):
            df_ohlc.loc[index, 'High'] = max(row['Close'], row['Open'], row['Low']) * 1.001
            count_of_ohlc_mistakes += 1
        if row['Volume'] < 0:
            df_ohlc.loc[index, 'Volume'] = abs(row['Volume'])
            count_of_ohlc_mistakes += 1

    # delete duplicates
    logging.debug('Duplicates found: %s', len(df_ohlc[df_ohlc.index.duplicated()]))
    df_ohlc = df_ohlc[~df_ohlc.index.duplicated()]

    df_ohlc.fillna(method='ffill', inplace=True)
    logging.debug('Missed candles added: %s', len(df_ohlc) - len(df_before_correction))

    return df_ohlc



def get_ohlc_cryptocompare_once(first_ticker, second_ticker, end_date=dt.datetime.now(),
                                aggregate=1, interval_key='day'):
    """ Retrieve limited bulk of ohlc cryptocurrency data from Cryptocompare.

    Args:

        first_ticker (str):
            Crypto symbol(BTC).
        second_ticker (str):
            Crypto symbol(USD).
        aggregate (int):
            How many points should be made into one
        interval_key (str):
            Time interval of data points
        end_date (datetime):
            Last moment in ohlc data

    Returns:
        df_ohlc (pandas.DataFrame):
            DF containing the opening price, high price, low price,
            closing price, and volume.

    Note:
        Data is limited(only 2000 point of data will be given)
    """
    limit = 2000
    df_ohlc = pd.DataFrame()
    interval_dict = {'minute': 'histominute', 'hour': 'histohour', 'day': 'histoday'}
    freq_dict = {'minute': '1M', 'hour': '1H', 'day': '1D'}
    end_date_unix = from_datetime_to_unix(end_date)

    url = 'https://min-api.cryptocompare.com/data/{}'.format(interval_dict[interval_key]) +\
        '?fsym={}'.format(first_ticker) +\
        '&tsym={}'.format(second_ticker) +\
        '&limit={}'.format(limit) +\
        '&aggregate={}'.format(aggregate) +\
        '&toTs={}'.format(str(end_date_unix))
    response = requests.get(url)
    resp_dict = json.loads(response.text)


    # parsing response dict to pieces
    if resp_dict["Response"] == "Success":
        data = resp_dict['Data']
        df_ohlc = pd.DataFrame(data)
        df_ohlc = columns_to_upper_case(df_ohlc)
        df_ohlc['Date'] = [dt.datetime.fromtimestamp(d) for d in df_ohlc.Time]
        df_ohlc['Volume'] = [v for v in df_ohlc.Volumeto]
        df_ohlc.set_index('Date', inplace=True)
        df_ohlc.index.name = 'Date'
        df_ohlc = correct_ohlc_df(df_ohlc, freq_dict[interval_key])

    elif resp_dict["Response"] == "Error":
        logging.error("There was an error in response from cryptocompare: %s", resp_dict)
    else:
        logging.error("Unknown response from cryptocompare: %s", resp_dict)

    return df_ohlc

def get_ohlc_cryptocompare(first_ticker, second_ticker, start_date,
                           end_date=dt.datetime.now(), **kwargs):
    """ Retrieves ohlc cryptocurrency data from Cryptocompare.

    Args:

        first_ticker (str):
            Crypto symbol(BTC).
        second_ticker (str):
            Crypto symbol(USD).
        start_date (datetime):
            First moment in ohlc data
        end_date (datetime):
            Optional.Last moment in ohlc data
        aggregate (int):
            Optional.How many points should be made into one
        interval_key (str):
            Optional.Time interval of data points


    Returns:
        df_total (pandas.DataFrame):
            DF containing the opening price, high price, low price,
            closing price, and volume.

    Note:
        This this loop for get_ohlc_cryptocompare_once
    """
    freq_dict = {'minute': '1M', 'hour': '1H', 'day': '1D'}
    df_total = get_ohlc_cryptocompare_once(first_ticker, second_ticker,
                                           end_date=end_date, **kwargs)
    new_start_date = df_total.index.min()
    while new_start_date > start_date:
        df_tmp = get_ohlc_cryptocompare_once(first_ticker, second_ticker,
                                             end_date=new_start_date, **kwargs)
        new_start_date = df_tmp.index.min()
        frames = [df_tmp, df_total]
        df_total = pd.concat(frames)
        df_total.drop_duplicates(inplace=True)
        time.sleep(10) # sort of gentle timeout for cryptocompare
    df_total = df_total[df_total.index >= start_date]
    if 'interval_key' in kwargs:
        df_total = correct_ohlc_df(df_total, freq_dict[kwargs['interval_key']])

    return df_total


# tryouts
if __name__ == "__main__":
    # vars for cryptocompare
    F_TICKER = 'BTC'
    S_TICKER = 'USD'

    END_DATE = dt.datetime(2018, 7, 1, 0, 0, 0)
    START_DATE = dt.datetime(2018, 3, 1, 0, 0, 0)

    OHLC_DF = get_ohlc_cryptocompare(F_TICKER, S_TICKER, START_DATE,
                                     end_date=END_DATE, interval_key='hour')

    print(OHLC_DF)



#!/usr/bin/env python3
"""Module to make package from folder"""
from pkg_resources import get_distribution, DistributionNotFound

try:
    # Change here if project is renamed and does not equal the package name
    DIST_NAME = 'stbt'
    __version__ = get_distribution(DIST_NAME).version
except DistributionNotFound:
    __version__ = 'latest'
finally:
    del get_distribution, DistributionNotFound



#!/usr/bin/env python3
"""Module with Strategy class to all
   backtest related manipulations"""
import logging
import itertools
import random
import pickle
import operator
import pandas as pd
from pandas.plotting import register_matplotlib_converters
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.dates as mdates
import seaborn as sns
import stbt.helpers as hf
register_matplotlib_converters()

# logging
###########################################################################
LOG_FORMAT = '%(asctime)s: %(filename)s: %(funcName)s: %(message)s'
FORMATTER = logging.Formatter(LOG_FORMAT, datefmt='%b %d %H:%M:%S')
SYSLOG = logging.StreamHandler()
SYSLOG.setFormatter(FORMATTER)

logger = logging.getLogger('simulator')
logger.addHandler(SYSLOG)
logger.setLevel(logging.INFO)
###########################################################################


class Strategy(object):
    """Class to do all backtesting, visualization and statistics calculation

    Note:
        data_df and weights_df should have same columns(name and len)

    Args:
        data_df (DataFrame):
            Close prices of instruments should be columns.
        weights_df (DataFrame):
            Money distribution for every day, same form with data_df
        pool_file (str):
            File to save pnl of created strategy
        cash (float64):
            Starting capital and returns multiplier

    Attributes:
        data (DataFrame):
            Close prices of instruments should be columns.
        weights (DataFrame):
            Money distribution for every day, same form with data
        cash (float64):
            Starting capital and returns multiplier
        pool_file (str):
            File to save pnl of created strategy
        pnl (DataFrame):
            Accumelated profit and loss of strategy, index is time
        stats_dict (dict):
            All the math statistics of strategy backtest
        data_mistakes_dict (dict):
            All the data inconsistencies
        stats_figure (figure):
            Statistics visualized
        strategy_figure (figure):
            Strategy visualized
        tests_figure (figure):
            Tests visualized
    """

    def __init__(self, data_df, weights_df, pool_file='strategy_pool.pickle', cash=1.0):
        """Constructor method"""
        self.data = data_df
        # scaling weights, sum of absolute values is one for every row
        self.weights = weights_df.div(
            weights_df.abs().sum(axis=1), axis=0).fillna(0)
        self.instruments = []  # not in use
        self.cash = cash  # not in use, because reasons
        self.pool_file = pool_file
        self.pnl = None
        self.stats_dict = {}
        self.mini_stats_dict = {}
        self.data_mistakes_dict = {
            'shape': 0,
            'index_type': 0,
            'duplicates': 0,
            'Nans': 0,
            'missed': 0,
            'dates_values': 0,
            'column_names': 0,
            'time_order': 0}
        self.stats_figure = None
        self.strategy_figure = None
        self.tests_figure = None

    def verify_data_integrity(self, frequency=None):
        """Method to check data passed in constructor for mistakes

        Args:
            frequency (str):
                Timeframe for data resampling 'D', 'W', 'M',
                if None - do not resample data

        """
        # lens
        if len(self.data) != len(self.weights):
            self.data_mistakes_dict['shape'] += 1
        if len(self.data.columns) != len(self.weights.columns):
            self.data_mistakes_dict['shape'] += 1

        # index type
        if not isinstance(self.data.index, pd.DatetimeIndex):
            self.data_mistakes_dict['index_type'] += 1
        if not isinstance(self.weights.index, pd.DatetimeIndex):
            self.data_mistakes_dict['index_type'] += 1

        # duplicates
        if len(self.weights[self.weights.index.duplicated()]) >= 1:
            self.data_mistakes_dict['duplicates'] += 1
        if len(self.data[self.data.index.duplicated()]) >= 1:
            self.data_mistakes_dict['duplicates'] += 1

        # NaNs
        for i in (len(self.data) - self.data.count()).values:
            if i != 0:
                self.data_mistakes_dict['Nans'] += 1
        for i in (self.weights.count() - len(self.weights)).values:
            if i != 0:
                self.data_mistakes_dict['Nans'] += 1
                inds = pd.isnull(self.weights).any(1).nonzero()[0]
                logger.debug('wrong indexes: %s', inds)

        # missed
        if frequency:
            # fill gaps in data
            df_reindexed = self.data.reindex(self.data.date_range(start=self.data.index.min(),
                                                                  end=self.data.index.max(),
                                                                  freq=frequency))

            df_reindexed.fillna(method='ffill', inplace=True)
            if len(df_reindexed) - len(self.data) >= 1:
                self.data_mistakes_dict['missed'] += 1

        # dates values
        if self.data.index[0] != self.weights.index[0]:
            self.data_mistakes_dict['dates_values'] += 1
        if self.data.index[-1] != self.weights.index[-1]:
            self.data_mistakes_dict['dates_values'] += 1

        # same columns
        if list(self.data.columns) != list(self.weights.columns):
            self.data_mistakes_dict['column_names'] += 1

        # time order
        if self.data.index[0] != self.data.index.min():
            self.data_mistakes_dict['time_order'] += 1
        if self.data.index[-1] != self.data.index.max():
            self.data_mistakes_dict['time_order'] += 1

        # mistakes assesment
        data_is_okay = True
        for key in self.data_mistakes_dict:
            if self.data_mistakes_dict[key] != 0:
                logger.error('There are mistakes in data!, self.data_mistakes_dict: %s',
                             self.data_mistakes_dict)
                data_is_okay = False
        if not data_is_okay:
            raise ValueError('Take a look at data passed to Strategy')
        else:
            logger.debug('Data in Strategy is okay, good to go')

    def backtest(self, delay=1, instruments_drop=None, commissions_const=0.0, capitalization=False,
                 start_date=None, end_date=None):
        """Method to calculate returns and pnl

        Args:
            delay (int):
                Time delay in applying weights to data
            instruments_drop (list):
                Columns with such names will be droped from data and weights
            commissions_const (float64):
                Fee paid for every transaction: 0.01 is 1% fee for every trade
            capitalization (Boolean):
                If money should be reinvested every time
            start_date (datetime):
                Date to start trading
            end_date (datetime):
                Date to end trading

        Returns:
            dict with pnl, returns, commissions dataframes
        """

        # initialize local data for simulate
        #########################################################
        simulate_data = self.data.copy()
        simulate_weights = self.weights.copy()
        #########################################################

        # filter instruments
        #########################################################
        if instruments_drop is None:
            pass
        else:
            simulate_data.drop(columns=instruments_drop, inplace=True)
            simulate_weights.drop(columns=instruments_drop, inplace=True)
        #########################################################

        # filter time
        #########################################################
        if (start_date is None) or (end_date is None):
            pass
        else:
            simulate_data = simulate_data[(simulate_data.index > start_date) & (
                simulate_data.index < end_date)]
            simulate_weights = simulate_weights[(simulate_weights.index > start_date) & (
                simulate_weights.index < end_date)]

        #########################################################

        # delay
        #########################################################
        if delay:
            simulate_weights = simulate_weights.shift(delay, fill_value=0)

        #########################################################

        # initialize everything
        #########################################################
        commissions = pd.DataFrame(
            np.zeros(len(simulate_data)), index=simulate_data.index, columns=['coms'])
        returns_df = pd.DataFrame()
        pnl = pd.DataFrame(np.zeros(len(simulate_data)), index=simulate_data.index,
                           columns=['_'.join(simulate_data.columns)])
        inst_sum_returns = pd.DataFrame(
            np.zeros(len(simulate_data)), index=simulate_data.index)
        #########################################################

        # calculate commissions
        #########################################################
        weights_diff_df = simulate_weights.diff()
        if commissions_const > 0.0:
            commissions = commissions.add(weights_diff_df.sum(axis=1), axis=0)
            commissions = abs(commissions) * commissions_const
            commissions.fillna(0, inplace=True)
        #########################################################

        # calculate returns
        #########################################################
        returns_df = simulate_data.pct_change()
        returns_df.fillna(0, inplace=True)
        #########################################################

        # calculate returns for all instruments with respect to weights and commissions
        #########################################################
        daily_returns_dist = ((returns_df) * simulate_weights)
        inst_sum_returns = inst_sum_returns.add(
            daily_returns_dist.sum(axis=1), axis=0)
        inst_sum_returns.columns = ['_'.join(simulate_data.columns)]
        inst_sum_returns = inst_sum_returns.subtract(
            commissions[['coms']].values)

        #########################################################

        # calculate pnl with respect to capitalization
        #########################################################
        if not capitalization:
            pnl = (inst_sum_returns * self.cash).cumsum() + self.cash

        else:
            fake_returns = inst_sum_returns.copy()
            fake_returns.iloc[0][0] = self.cash
            pnl[pnl.columns[0]] = ((fake_returns + 1).cumprod())

        #########################################################
        logger.debug('Strategy was backtested')

        return {
            'pnl': pnl,
            'returns': inst_sum_returns,
            'coms': commissions,
            'capitalization': capitalization,
            'delay': delay,
            'commissions_const': commissions_const
        }

    def calculate_sim_stats(self, pnl, returns, plot=False):
        """Method to calculate vatious statistics of simulation

        Args:
            pnl (DataFrame):
                Accumelated profit and loss of strategy, index is time
            returns (DataFrame):
                Strategy returns for every day, index is time

        Returns:
            dict sim_stats_dict with great deal of stats

        Note:
            Also creates stats_figure attribute
        """

        sim_stats_dict = {
            'start_date': str(returns.index[0]),
            'end_date': str(returns.index[-1]),
            'Sharpe': 0,
            'Sharpe_1d': 0,
            'Sharpe_30d': 0,
            'Sharpe_90d': 0
        }

        # sharpe calculation:
        sim_stats_dict['Sharpe'] = hf.get_sharpe(returns)

        # pnl correlation with data
        tmp_corr_df = pnl.join(self.data, lsuffix='_x').corr(
        ).iloc[:, 0].drop(pnl.columns[0], axis=0)
        sim_stats_dict['Pnl_inst_Correlation'] = {tmp_corr_df.argmax(): round(tmp_corr_df.max(), 2),
                                                  tmp_corr_df.argmin(): round(tmp_corr_df.min(), 2)}
        # print(tmp_corr_df)
        del tmp_corr_df
        # pnl.fillna(0).corrwith(self.data)

        # 1day sharpe
        df_resampled_sharpe_1d = returns.resample('1d').apply(hf.get_sharpe)
        sim_stats_dict['Sharpe_1d'] = round(
            df_resampled_sharpe_1d.mean()[0], 1)

        # 30days sharpe
        df_resampled_sharpe_30d = returns.resample('30d').apply(hf.get_sharpe)
        sim_stats_dict['Sharpe_30d'] = round(
            df_resampled_sharpe_30d.mean()[0], 1)

        # 30days sharpe
        df_resampled_sharpe_90d = returns.resample('90d').apply(hf.get_sharpe)
        sim_stats_dict['Sharpe_90d'] = round(
            df_resampled_sharpe_90d.mean()[0], 1)

        # total returns
        sim_stats_dict['Total_returns'] = str(
            round(returns.sum()[0] * 100, 2)) + '%'

        # avg_returns per period
        sim_stats_dict['Avg_returns'] = str(
            round(returns.mean()[0] * 100, 2)) + '%'

        # avg_returns per day
        returns_resampled_1d = returns.resample('1d').sum()
        sim_stats_dict['Avg_returns_1d'] = str(
            round(returns_resampled_1d.mean()[0] * 100, 2)) + '%'

        # avg_returns per month
        returns_resampled_30d = returns.resample('30d').sum()
        sim_stats_dict['Avg_returns_30d'] = str(
            round(returns_resampled_30d.mean()[0] * 100, 2)) + '%'

        # Max Drawdawn
        dd_tuple = hf.get_max_drawdown(returns.iloc[:, 0])
        sim_stats_dict['Max_Drawdown'] = str(
            round(abs(dd_tuple[0]) * 100, 1)) + '%'

        # Daily Turnover
        weights_diff = abs(self.weights.diff())

        weights_diff_resampled = weights_diff.resample('1d').sum()
        max_turnover = {'%': round(weights_diff_resampled.mean().max()*100, 1),
                        'ticker': weights_diff_resampled.mean().idxmax()

                        }
        turnover = str(
            round(weights_diff_resampled.mean().sum() * 100, 1)) + '%'

        sim_stats_dict['Turnover_max'] = max_turnover
        sim_stats_dict['Turnover_1d'] = turnover

        turnover_resampled = weights_diff_resampled.resample(
            '30d').mean() * 100

        # MAX CORR
        sim_stats_dict['Max_corr'] = self.get_max_corr(pnl)

        # plot stats
        if plot:
            self.stats_figure = plt.figure(tight_layout=True)

            axis = plt.subplot2grid((12, 1), (0, 0), rowspan=2, colspan=1)
            axis.plot(returns_resampled_30d.index.values,
                      returns_resampled_30d.values)
            axis.plot(returns_resampled_30d.index.values,
                      np.zeros(len(returns_resampled_30d)), 'r--')
            axis.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
            axis.xaxis.set_major_locator(mticker.MaxNLocator(5))
            plt.title('Avg_returns_30d')

            axis2 = plt.subplot2grid((12, 1), (3, 0), rowspan=2, colspan=1)
            axis2.plot(df_resampled_sharpe_30d.index.values,
                       df_resampled_sharpe_30d.values)
            axis2.plot(df_resampled_sharpe_30d.index.values,
                       np.zeros(len(df_resampled_sharpe_30d)), 'r--')
            axis2.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
            axis2.xaxis.set_major_locator(mticker.MaxNLocator(5))
            plt.title('Sharpe_30d')

            axis3 = plt.subplot2grid((12, 1), (6, 0), rowspan=2, colspan=1)
            axis3.plot(df_resampled_sharpe_90d.index.values,
                       df_resampled_sharpe_90d.values)
            axis3.plot(df_resampled_sharpe_90d.index.values,
                       np.zeros(len(df_resampled_sharpe_90d)), 'r--')
            axis3.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
            axis3.xaxis.set_major_locator(mticker.MaxNLocator(5))
            plt.title('Sharpe_90d')

            axis4 = plt.subplot2grid((12, 1), (9, 0), rowspan=2, colspan=1)
            axis4.plot(turnover_resampled.index.values,
                       turnover_resampled.values)
            axis4.plot(turnover_resampled.index.values,
                       np.zeros(len(turnover_resampled)), 'r--')
            axis4.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
            axis4.xaxis.set_major_locator(mticker.MaxNLocator(5))
            plt.title('Turnover_daily_30d_mean')

        logger.debug('Statistics for strategy were calculated:')

        self.mini_stats_dict['Sharp'] = sim_stats_dict['Sharpe']
        self.mini_stats_dict['Avg_returns_1d'] = sim_stats_dict['Avg_returns_1d']
        self.mini_stats_dict['Max_Drawdown'] = sim_stats_dict['Max_Drawdown']
        self.mini_stats_dict['Turnover'] = turnover
        self.mini_stats_dict['Turnover_max'] = max_turnover

        return sim_stats_dict

    def plot_sim_results(self, pnl):
        """Method to visualize previous backtest

        Args:
            pnl (DataFrame):
                Accumelated profit and loss of strategy, index is time

        Note:
            Creates strategy_figure attribute
        """

        self.strategy_figure = plt.figure()

        ax1 = plt.subplot2grid((12, 1), (0, 0), rowspan=3, colspan=1)
        ax1.plot(self.data.index, self.data)
        ax1.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
        ax1.xaxis.set_major_locator(mticker.MaxNLocator(5))
        plt.title('Instruments Price')
        for xlabel_i in ax1.get_xticklabels():
            xlabel_i.set_visible(False)
        plt.legend(self.data.columns)
        plt.yscale('log')

        ax2 = plt.subplot2grid((12, 1), (4, 0), rowspan=5, colspan=1)
        ax2.plot(pnl.index.values, pnl.values)
        ax2.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
        ax2.xaxis.set_major_locator(mticker.MaxNLocator(5))
        plt.title('PnL')
        for xlabel_i in ax2.get_xticklabels():
            xlabel_i.set_visible(False)

        ax3 = plt.subplot2grid((12, 1), (10, 0), rowspan=2, colspan=1)
        ax3.plot(self.weights.index.values, self.weights.values)
        ax3.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
        ax3.xaxis.set_major_locator(mticker.MaxNLocator(5))
        plt.title('Weights')

        logger.debug('Graph with backtest results was created')

    def run_tests(self, delay1=1, delay2=2, delay3=3, delay4=30, delay5=60):
        """Method to check strategy robusness against time and comissions

        Note:
            Creates tests_figure attribute
        """
        list_of_res_dicts = []
        tests = [
            {'delay': delay1},
            {'delay': delay2},
            {'delay': delay3},
            {'delay': delay4},
            {'delay': delay5},
            # {'delay': 2, 'commissions_const': 0.001}
        ]

        self.tests_figure = plt.figure()
        axis = plt.subplot2grid((12, 1), (0, 0), rowspan=12, colspan=1)

        test_number = 0
        for test in tests:
            list_of_res_dicts.append(self.backtest(**test))
            axis.plot(self.weights.index,
                      list_of_res_dicts[-1]['pnl'], label='{}'.format(hf.get_label_from_dict(test)))
            test_number += 1
        # print(list_of_res_dicts)
        axis.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
        axis.xaxis.set_major_locator(mticker.MaxNLocator(5))
        plt.title('Tests_pnls')
        plt.legend()

        return list_of_res_dicts

    def run_all(self, delay=1, verify_data_integrity=True, instruments_drop=None,
                commissions_const=0, capitalization=False, start_date=None, end_date=None,
                delay1=1, delay2=2, delay3=3, delay4=30, delay5=60, plot='tests'):
        """Method to get all info about strategy(run all methods)

        Args:
            delay (int):
                Time delay in applying weights to data
            instruments_drop (list):
                Columns with such names will be droped from data and weights
            commissions_const (float64):
                Fee paid for every transaction: 0.01 is 1% fee for every trade
            capitalization (Boolean):
                If money should be reinvested every time
            start_date (datetime):
                Date to start trading
            end_date (datetime):
                Date to end trading
        """
        if verify_data_integrity:
            self.verify_data_integrity()
        results_dict = self.backtest(instruments_drop=instruments_drop,
                                     commissions_const=commissions_const,
                                     capitalization=capitalization,
                                     delay=delay,
                                     start_date=start_date,
                                     end_date=end_date)
        if plot == 'all':
            self.plot_sim_results(results_dict['pnl'])
        self.pnl = results_dict['pnl']
        if plot == 'all':
            self.stats_dict = self.calculate_sim_stats(
                results_dict['pnl'], results_dict['returns'], plot=True)
        else:
            self.stats_dict = self.calculate_sim_stats(
                results_dict['pnl'], results_dict['returns'], plot=False)
        logger.debug(str(self.stats_dict))
        if plot == 'all' or plot == 'tests':
            self.run_tests(delay1, delay2, delay3, delay4, delay5)

    def get_pnls_pool(self):
        """Method to read all pnls from self.pool_file"""
        with open(self.pool_file, 'rb') as filename:
            pnls_df = pickle.load(filename)

        return pnls_df

    def add_to_pnls_pool(self, pnl_df=None, name=str(random.randint(1, 9999)) + '_pnl'):
        """Method to add pnls to self.pool_file"""
        if not pnl_df:
            pnl_df = self.pnl
        pnl_df.rename(columns={pnl_df.columns[0]: name}, inplace=True)
        try:
            pnls_df = self.get_pnls_pool()
            if len(pnl_df) == len(pnls_df):
                pnls_df = pnls_df.join(pnl_df)
                with open(self.pool_file, 'wb') as filename:
                    pickle.dump(pnls_df, filename)
            else:
                logger.error(
                    'Length of dfs is inconsistent: cant save such pnls!')

        except FileNotFoundError:
            pnls_df = pnl_df
            with open(self.pool_file, 'wb') as filename:
                pickle.dump(pnls_df, filename)

        except ValueError:
            logger.error('''You are trying to add pnl which already exists!
(change column name to add it)''')

        return pnls_df

    def get_pool_heatmap(self):
        """Method to visualize self.pool_file"""
        pnls_df = self.get_pnls_pool()
        corr = pnls_df.corr()

        figure = plt.figure()
        sns.heatmap(corr, annot=True)
        plt.title('Correlation heatmap')

        return figure, corr

    def get_max_corr(self, pnl):
        """Method to get highest correlation with pnl from self.pool_file"""
        corr_dict = {}
        try:
            pnls_df = self.get_pnls_pool()
            time_delta = pnl.index[1] - pnl.index[0]
            if time_delta != pd.Timedelta(1, 'h'):
                pnl = hf.resample(pnl, 'H')

            if len(pnl) > len(pnls_df):
                zero_df = pd.DataFrame(
                    np.zeros(len(pnls_df)), index=pnls_df.index, columns=pnl.columns)
                pnl = zero_df + pnl
                pnl = pnl.dropna()
            if len(pnl) < len(pnls_df):
                zero_df = pd.DataFrame(
                    np.zeros(len(pnls_df)), index=pnls_df.index, columns=pnl.columns)
                pnl = zero_df + pnl
                pnl = pnl.ffill()

            for column in pnls_df:
                corr_dict[column] = pnl.corrwith(pnls_df[column]).values[0]

            top_key = max(corr_dict.items(), key=operator.itemgetter(1))[0]
            res_list = [top_key, corr_dict[top_key]]

        except BaseException as err:
            logger.debug(err)
            res_list = ['0', 0]

        return res_list


# Special functions:

def get_correlation(list_of_pnls, plot=True):
    """Function to get correlation heatmap

    Args:
        list_of_pnls (list):
            List with dataframes of pnls

    Returns:
        corr (DataFrame):
            Matrix of correalations
        figure (figure):
            Seaborn heatmap
    """

    pnl_df = pd.DataFrame()
    for counter, pnl in enumerate(list_of_pnls):
        pnl.rename(
            columns={pnl.columns[0]: "{}".format(counter)}, inplace=True)
        if pnl_df.empty:
            pnl_df = pnl
        else:
            pnl_df = pnl_df.join(pnl)

    corr = pnl_df.corr()

    figure = None
    if plot:
        figure = plt.figure()
        sns.heatmap(corr, annot=True)
        plt.title('Correlation heatmap')

    return corr, figure


def loop_over_alphas(alphas_to_run, data):
    '''simple way to run alphas one by one
    alphas_to_run = \
    [
        {'alpha': ta.event_high_volume,
        'args': ['volume', 'close'],
        'kwargs': {'volume_threshold': [5, 10, 15, 30],
                    'close_threshold': [0.995, 1, 1.002, 1.005, 1.1],
                    'lookback': [480],
                    'live_time_minutes': [30, 60, 120, 180]
                }
        }
    ]
    '''
    for alpha in alphas_to_run:
        total_params_list = []
        for kwarg in alpha['kwargs']:
            total_params_list.append(alpha['kwargs'][kwarg])

        param_combinations = list(itertools.product(*total_params_list))
        for params in param_combinations:
            tmp_kwargs = {}
            i = 0
            # create kwargs
            for kwarg in alpha['kwargs']:
                tmp_kwargs[kwarg] = params[i]
                i += 1
            print('#'*100)
            print(f"Alpha: {alpha['alpha'].__name__}")
            print(f"Params: {tmp_kwargs}")
            # create args
            for universe in data:
                print(f"Universe: {universe}")
                tmp_args = [data[universe][arg] for arg in alpha['args']]
                weights = alpha['alpha'](*tmp_args, **tmp_kwargs)
                strategy = Strategy(data[universe]['close'], weights, cash=100)
                strategy.run_all(delay=2, verify_data_integrity=True, instruments_drop=None,
                                 commissions_const=0, capitalization=False,
                                 delay1=2, delay2=5, delay3=10, delay4=30, delay5=60, plot='tests')
                plt.show()
                print(f"Stats: {strategy.mini_stats_dict}")


def full_backtest_universe_prod_alphas(alpha_dict: dict,
                                       data: dict,
                                       universe: str = 'moex_stocks') -> None:
    '''Plot all tests for alphas of the same universe
    '''
    print(f"Universe: {universe}")
    for alpha_name in alpha_dict:
        if universe in alpha_dict[alpha_name]:
            tmp_alpha = alpha_dict[alpha_name]['alpha']
            for alpha_params in alpha_dict[alpha_name][universe]:
                print('#'*100)
                print(f"Alpha: {tmp_alpha.__name__}")
                print(f"Params: {alpha_params}")

                weights = tmp_alpha(data[universe]['all'], **alpha_params)
                strategy = Strategy(data[universe]['close'], weights, cash=100)
                strategy.run_all(delay=2, verify_data_integrity=True, instruments_drop=None,
                                 commissions_const=0, capitalization=False,
                                 delay1=2, delay2=5, delay3=10, delay4=30, delay5=60, plot='all')
                plt.show()
                print(f"Stats: {strategy.mini_stats_dict}")
